﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;

namespace Kaleida.ServiceMonitor.Model.Layout
{
    class NestedTilesLayoutStrategy : IScriptStateLayoutStrategy
    {
        public IList<ScriptNodeArea> CalculateLayout(IScriptNode rootNode, Rectangle availableSpace)
        {
            return CalculateLayout(rootNode, availableSpace, true);
        }

        private static IList<ScriptNodeArea> CalculateLayout(IScriptNode node, Rectangle availableSpace, bool splitHorizontally)
        {
            if (!node.Children.Any())
                return new List<ScriptNodeArea> { new ScriptNodeArea(node, availableSpace) };


            var rectangles = new List<ScriptNodeArea>();
            for (int i = 0; i < node.Children.Count; i++)
            {
                var child = node.Children[i];

                var allocation = CalculateShare(availableSpace, node.Children.Count, i, splitHorizontally);

                rectangles.AddRange(CalculateLayout(child, allocation, !splitHorizontally));
            }
            return rectangles;
        }

        private static Rectangle CalculateShare(Rectangle availableSpace, int total, int index, bool splitHorizontally)
        {
            return splitHorizontally
                       ? CalculateHorizontalShare(availableSpace, total, index)
                       : CalculateVerticalShare(availableSpace, total, index);
        }

        private static Rectangle CalculateHorizontalShare(Rectangle availableArea, int total, int index)
        {
            var bounds = GetBounds(index, total, availableArea.Height);

            return new Rectangle(availableArea.Left, availableArea.Top + bounds.Item1, availableArea.Width, bounds.Item2 - bounds.Item1);
        }

        private static Rectangle CalculateVerticalShare(Rectangle availableArea, int total, int index)
        {
            var bounds = GetBounds(index, total, availableArea.Width);
            return new Rectangle(availableArea.Left + bounds.Item1, availableArea.Top, bounds.Item2 - bounds.Item1, availableArea.Height);
        }

        private static Tuple<int,int> GetBounds(int index, int total, int available)
        {
            var start = index*(available/total);
            var end = index < total - 1
                          ? (index + 1)*(available/total)
                          : available;

            return new Tuple<int, int>(start, end);
        }
    }
}
