grammar ScriptGrammar;

options {
  language=CSharp2;
  output=AST;
}

tokens {
  GROUP;
  COMMAND;
  DIRECTIVE;
  COMMENT;
}

@parser::namespace { Kaleida.ServiceMonitor.Model.Parsing }
@lexer::namespace { Kaleida.ServiceMonitor.Model.Parsing }

file:
  scriptGroup EOF
  ;

scriptGroup:
  ( structuredCommand | structuredDirective | structuredComment | blankLine | emptyScriptGroup | bracketedScriptGroup )*
  ;

emptyScriptGroup:
  (GroupStartBrace GroupEndBrace ) -> GROUP
  ;

bracketedScriptGroup:
  (GroupStartBrace scriptGroup GroupEndBrace ) -> ^(GROUP scriptGroup)
  ;

structuredCommand:
 command -> ^(COMMAND command)
 ;

structuredDirective:
 directive -> ^(DIRECTIVE directive)
 ;
 
 structuredComment:
 comment -> ^(COMMENT comment)
 ;

command:
  actionAndArgs impersonationInstruction? actionAndArgs? (LineBreak! | EOF!) 
  ;

directive:
  PrefixedDirective^ argList (LineBreak! | EOF!) 
  ;

comment:
  LineComment (LineBreak! | EOF!) 
  ;

actionAndArgs:
  Identifier^ argList
  ;

impersonationInstruction:
  ImpersonationKeyword^ QuotedString Comma! QuotedString 
  ;

argList:
  ( QuotedString ( Comma! QuotedString )* )?
  ;

blankLine:
  WhiteSpace* LineBreak
  ;

WhiteSpace:
  (' ' | '\t'){$channel=HIDDEN;}
  ;

LineBreak:
  ('\r' | '\n')+
  ;

Comma:
  ','
  ;

PrefixedDirective:
  DirectivePrefix Identifier
  ;

ImpersonationKeyword:
  'runas'
  ;

Identifier:
  ('A'..'Z' | 'a'..'z') ('A'..'Z' | 'a'..'z' | '0'..'9' | '-' )*
  ;

QuotedString:
  Quote ('""' | ~'"')* Quote
  ;
  
LineComment:
  '//' ~( '\r' | '\n' )*
  ;

Quote:
  '"'
  ;

DirectivePrefix:
  '#'
  ;

GroupStartBrace:
  '{'
  ;

GroupEndBrace:
  '}'
  ;

Invalid:
  . 
  ;