using System;
using System.Text.RegularExpressions;
using Kaleida.ServiceMonitor.Framework;
using Kaleida.ServiceMonitor.Framework.Responses;

namespace Kaleida.ServiceMonitor.Operations.ResponseHandlers
{
    public class MustMatch : ResponseHandler
    {
        private readonly Regex regex;

        public MustMatch(string regex)
        {
            this.regex = new Regex(regex, RegexOptions.Compiled);
        }

        public override string Description
        {
            get { return string.Format("expect the response to match regular expression '{0}'", regex); }
        }

        public override string ProcessResponse(object response)
        {
            if (response is IStringContainer)
            {
                var actualContent = ((IStringContainer)response).Content;
                TestAsString(actualContent);
                return GetReturnString(actualContent);
            }

            var actualString = response.ToString();
            TestAsString(actualString);
            return GetReturnString(actualString);
        }

        private static string GetReturnString(string actualString)
        {
            return string.Format("Regex matched document of length {0} character{1}", actualString.Length, actualString.Length == 1 ? "" : "s");
        }

        private void TestAsString(string actualString)
        {
            if (!regex.IsMatch(actualString))
                throw new InvalidOperationException(string.Format("Expected response to match regular expression '{0}'", regex));
        }
    }
}